package URI::Fast::IRI;

our @ISA = qw(URI::Fast);

=head1 NAME

URI::Fast::IRI - Support for IRIs in URI::Fast

=head1 DESCRIPTION

This module is part of L<URI::Fast> and is not directly instantiated. Instead,
use L<URI::Fast/iri>.

=head1 AUTHOR

Jeff Ober <sysread@fastmail.fm>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2018 by Jeff Ober.

This is free software; you can redistribute it and/or modify it under the same
terms as the Perl 5 programming language system itself.

=cut

1;
